<?php
/**
 * User: nester_a
 * Date: 25.06.14
 */

namespace LMS\Libraries\Database;

use JDatabaseQuery;
use JDatabaseQueryElement;

class Query extends \JDatabaseQuery implements \JDatabaseQueryLimitable
{
	/**
	 * @var \JDatabaseQuery
	 */
	private $decoratedQuery;

	protected $offset;

	protected $limit;

	private $joinedTables = array();

	/**
	 * @var \LMS\Libraries\Base\Table
	 */
	private $table;

	public function __construct(\JDatabaseQuery $query, Interfaces\IDriver $db = null)
	{
		$this->decoratedQuery = $query;
		parent::__construct($db);
	}

	public function setTable(\LMS\Libraries\Base\Table $table)
	{
		$this->table = $table;
	}

	/**
	 * @return \LMS\Libraries\Base\Table
	 */
	public function getTable()
	{
		return $this->table;
	}



	public function select($columns)
	{
		if(is_null($this->table)){
			throw new \InvalidArgumentException('Please run from() before select()');
		}

		$columns = $this->processColumns($columns);
		return parent::select($columns);
	}

	public function from($table, $subQueryAlias = null)
	{
		if(!$table instanceof \LMS\Libraries\Base\Table){
			throw new \InvalidArgumentException('In from() clause should be provided \LMS\Libraries\Base\Table class. Please correct');
		}

		$this->setTable($table);

		return parent::from($this->table->getTableName() . ' as ' . $this->db->quoteName($this->table->getTableAlias()));
	}

	public function where($conditions, $glue = 'AND')
	{
		if(is_null($this->table)){
			throw new \InvalidArgumentException('Please run from() before select()');
		}

		$conditions = $this->processConditions($conditions);
		return parent::where($conditions, $glue);
	}

	public function delete($table = null)
	{
		if(!$table instanceof \LMS\Libraries\Base\Table){
			throw new \InvalidArgumentException('In delete() clause should be provided \LMS\Libraries\Base\Table class. Please correct');
		}

		$this->setTable($table);
		parent::delete();
		return parent::from($this->table->getTableName());
	}

	public function update($table)
	{
		if(!$table instanceof \LMS\Libraries\Base\Table){
			throw new \InvalidArgumentException('In update() clause should be provided \LMS\Libraries\Base\Table class. Please correct');
		}

		$this->setTable($table);

		return parent::update($table->getTableName()); // TODO: Change the autogenerated stub
	}


	public function setLimit($limit = 0, $offset = 0)
	{
		$this->limit = $limit;
		$this->offset = $offset;
		if($this->decoratedQuery instanceof \JDatabaseQueryLimitable){
			$this->decoratedQuery->setLimit($limit, $offset);
		}
	}

	public function processLimit($query, $limit, $offset = 0)
	{
		return $this->decoratedQuery instanceof \JDatabaseQueryLimitable ? $this->decoratedQuery->processLimit($query, $limit, $offset) : $query;
	}

	/**
	 * @param $conditions
	 * @return array
	 */
	private function processConditions($conditions)
	{
		if(!is_array($conditions)){
			$conditions = array($conditions);
		}

		foreach($conditions as &$condition){
			preg_match_all('/([0-9A-Za-z\-_]+\.[0-9A-Za-z\-_\.]+)/', $condition, $fields);
			foreach($fields[1] as $field){
				$condition = str_replace($field, $this->processWhereField($field, $this->table), $condition);
			}
		}
		return $conditions;
	}

	/**
	 * @param $columns
	 * @return array
	 */
	private function processColumns($columns)
	{
		$columns = $this->prepareColumns($columns);
		foreach($columns as &$column){
			$column = $this->processField($column, $this->table);
		}
		return $columns;
	}


	private function processWhereField($field, $table)
	{
		$oldField = $field;
		preg_match_all('/([0-9A-Za-z\-_]+\.[0-9A-Za-z\-_\.]+)/', $field, $matches);

		if(!empty($matches[1])){
			$oldField = $matches[1][0];
		}

		$explodedField = explode('.', $oldField);

		if(count($explodedField) == 2){
			return $this->db->quoteName($explodedField[0]).'.'.$table->getField($explodedField[1]);
		}

		$relationName = $explodedField[1];
		$relation = $table->getRelation($relationName);

		$explodedField = $this->prepareJoin($explodedField, $table);

		$newField = $this->processWhereField(implode('.', $explodedField), $relation['table']);
		return str_replace($oldField, $newField, $field);
	}

	/**
	 * TODO: need to be refactored. Seems too complicated
	 *
	 * @param string $field
	 * @param \LMS\Libraries\Base\Table $table
	 * @return mixed
	 */
	private function processField($field, $table, $needAlias = true)
	{
		$oldField = $field;
		preg_match_all('/([0-9A-Za-z\-_]+\.[0-9A-Za-z\-_\.\*]+)/', $field, $matches);		

		if(strpos($field, ' as ') !== false || strpos($field, '(') !== false || !$needAlias){
			$needAlias = false;
		}

		if(!empty($matches[1])){
			$oldField = $matches[1][0];
		}

		$explodedField = explode('.', $oldField);
		if(count($explodedField) <= 1){
			throw new \InvalidArgumentException('Please provide table alias for column "'.$field.'"');
		}

		if(count($explodedField) == 2){
			if(!$needAlias){
				return $field;
			}
			if($explodedField[1] == '*'){
				$fields = '';
				foreach($table->getFields() as $field){
					$fields .= $explodedField[0].".".$field->Field .' as '. $table->getAlias($field->Field).', ';
				}
				return trim($fields, ', ');
			}

			return $explodedField[0].'.'.$table->getField($explodedField[1]) . ' as ' . $table->getAlias($explodedField[1]);
		}

		$relationName = $explodedField[1];
		$relation = $table->getRelation($relationName);

		$explodedField = $this->prepareJoin($explodedField, $table);

		$newField = $this->processField(implode('.', $explodedField), $relation['table'], $needAlias);
		return str_replace($oldField, $newField, $field);

	}

	/**
	 * @param string $condition
	 * @param \LMS\Libraries\Base\Table $parentTable
	 * @param \LMS\Libraries\Base\Table $childTable
	 */
	private function prepareRelationCondition($condition, $parentTable, $childTable, $parentAlias, $childAlias)
	{
		$condition = $childTable->getTableName() . ' ' . $this->db->quoteName($childAlias) . ' ON ' . $condition;

		preg_match_all('/{([0-9A-Za-z_\-\.]+)}/', $condition, $matches);
		foreach($matches[1] as $match){
			$parts = explode('.', $match);
			if($parts[0] == 'parent'){
				$field = $this->db->quoteName($parentAlias) . '.' . $parentTable->getField($parts[1]);
			}else{
				$field = $this->db->quoteName($childAlias) . '.' . $childTable->getField($parts[1]);
			}
			$condition = str_replace('{' . $match . '}', $field, $condition);
		}
		return $condition;
	}

	/**
	 * @param $columns
	 * @return array
	 */
	private function prepareColumns($columns)
	{
		if(!is_array($columns)){
			$columns = array($columns);
		}

		$newColumns = array();
		foreach($columns as $column){
			$explodedColumn = explode(',', $column);
			foreach($explodedColumn as $part){
				$newColumns[] = trim($part);
			}
		}

		return $newColumns;
	}

	/**
	 * @param $explodedField
	 * @param $table
	 * @return mixed
	 */
	private function prepareJoin($explodedField, $table)
	{
		$relationName = $explodedField[1];
		$relation = $table->getRelation($relationName);

		$parentAlias = $explodedField[0];

		if(!in_array($relation['table']->getTableName() . '_' . $relationName , $this->joinedTables)){
			$joinAlias = $parentAlias .  '_' .  $relationName;
			$this->join($relation['type'], $this->prepareRelationCondition($relation['condition'], $table, $relation['table'], $parentAlias, $joinAlias));
			$this->joinedTables[] = $relation['table']->getTableName(). '_' . $relationName;
		}

		array_shift($explodedField);
		$explodedField[0] = $parentAlias .  '_' .  $relationName;
		return $explodedField;
	}


}