<?php
/**
 * User: nester_a
 * Date: 24.07.14
 */

namespace LMS\Tests\Base;

use LMS\Libraries\ServiceLocator;
use LMS\Tests\Libs\DatabaseTestCase;
use LMS\Tests\Data\Tables;

class RepositoryTest extends DatabaseTestCase
{
	/**
	 * @var \LMS\Libraries\Base\Repository
	 */
	private $repository;

	/**
	 * @var \LMS\Libraries\Base\Table
	 */
	private $table;



	public function setUp()
	{
		parent::setUp();

		$db = ServiceLocator::get('db');

		ServiceLocator::setMocks(array(
			'db' => $db
		));

		$this->repository = new \LMS\Libraries\Base\Repository(array('db' => $db));
		$this->table = new \LMS\Tests\Data\Tables\TestTable1($db);
		$this->repository->setTable($this->table);
	}



	/**
	 *
	 */
	public function testSimpleFind()
	{
		$results = $this->repository->find(array($this->table->getTableAlias() . '.id'));
		$this->assertEquals(7, count($results));
		for($i=0; $i<count($results); $i++){
			$this->assertAttributeEquals($i+1, 'id', $results[$i]);
		}
	}

	/**
	 *
	 */
	public function testSimpleFindOrder()
	{
		$results = $this->repository->find(
			array($this->table->getTableAlias() . '.id'),
			array(),
			array('order' => $this->table->getTableAlias()  . '.name DESC')
		);

		$this->assertEquals(7, count($results));
		for($i=0; $i<count($results)-1; $i++){
			$this->assertTrue($results[$i]->table_name >= $results[$i+1]->table_name);
		}
	}

	/**
	 *
	 */
	public function testSimpleFindGroup()
	{
		$distinctResults = $this->repository->find(
			array('DISTINCT('.$this->table->getTableAlias() . '.name) as table_name')
		);

		$results = $this->repository->find(
			array($this->table->getTableAlias() . '.name'),
			array(),
			array('group' => $this->table->getTableAlias() . '.name')
		);

		$this->assertEquals(count($distinctResults), count($results));
		for($i=0; $i<count($results); $i++){
			$this->assertEquals($distinctResults[$i]->table_name, $results[$i]->table_name);
		}
	}

	/**
	 *
	 */
	public function testFindWithRelations()
	{
		$results = $this->repository->find(array($this->table->getTableAlias() . '.key_id', $this->table->getTableAlias() . '.child1.id',  $this->table->getTableAlias() . '.child1.name'));
		$this->assertEquals(7, count($results));
		foreach($results as $result){
			$this->assertEquals($result->key_id, $result->id);
			$this->assertContains('_name'.$result->key_id, $result->name);
		}
	}

	/**
	 *
	 */
	public function testSimpleFindWithSimpleWhere()
	{
		$results = $this->repository->find(array($this->table->getTableAlias() . '.id'), array('id <= 4'));
		foreach($results as $result){
			$this->assertAttributeLessThanOrEqual(4, 'id', $result);
		}
	}

	/**
	 *
	 */
	public function testSimpleFindWithSimpleWhereWithAlias()
	{
		$results = $this->repository->find(array($this->table->getTableAlias() . '.name'), array($this->table->getTableAlias() .'.table_name LIKE "%test1%"'));
		foreach($results as $result){
			$this->assertAttributeContains('test1', 'table_name', $result);
		}
	}

	/**
	 *
	 */
	public function testSimpleFindWithWhere()
	{
		$results = $this->repository->find(array($this->table->getTableAlias() . '.*'), array('id <= 5', 'key_id > 2'));
		foreach($results as $result){
			$this->assertAttributeLessThanOrEqual(5, 'id', $result);
			$this->assertAttributeGreaterThan(2, 'key_id', $result);
			$this->assertContains('_name'.$result->id, $result->table_name);
		}
	}

	/**
	 *
	 */
	public function testFindWithRelationsAndWhere()
	{
		$results = $this->repository->find(array($this->table->getTableAlias() . '.*', $this->table->getTableAlias() . '.child1.name as child1_name' , $this->table->getTableAlias() . '.child1.child2.id as child_id'), array($this->table->getTableAlias() . '.child1.child2.id <= 4'));
		foreach($results as $result){
			$this->assertContains('_name'.$result->key_id, $result->child1_name);
			$this->assertAttributeLessThanOrEqual(4, 'child_id', $result);
		}
	}

	/**
	 *
	 */
	public function testGetTable()
	{
		$this->assertEquals($this->table, $this->repository->getTable());
	}

	/**
	 *
	 */
	public function testGetByID()
	{
		$result = $this->repository->getByID(1);
		$this->assertEquals(1, $result->id);
		$columns = $this->table->getColumns(true);

		foreach($columns as $column){
			$this->assertTrue(isset($result->$column));
		}
	}

	/**
	 *
	 */
	public function testGetByFilter()
	{

		$filter = array(
			'fields' => array(
				$this->table->getTableAlias() . '.id',
				$this->table->getTableAlias() . '.name',
			),
			'id' => '1',
			'namePattern' => '%test%',
		);

		$stubRepository = $this->getStubbedRepository();
		$results = $stubRepository->getByFilter($filter);
		foreach($results as $result){
			$this->assertAttributeContains('test', 'table_name', $result);
			$this->assertAttributeGreaterThan(1, 'id', $result);
		}
	}

	/**
	 *
	 */
	public function testGetByOrder()
	{

		$filter = array(
			'fields' => array(
				$this->table->getTableAlias() . '.id',
				$this->table->getTableAlias() . '.name',
			),
			'namePattern' => '%test%',
			'list.ordering' => $this->table->getTableAlias() . '.name',
			'list.direction' => 'DESC'
		);

		$results = $this->repository->getByFilter($filter);
		$count = count($results);
		for($i=0; $i<$count-1; $i++){
			$this->assertAttributeContains('test', 'table_name', $results[$i]);
			$this->assertTrue($results[$i]->table_name >= $results[$i+1]->table_name);
		}

		$this->assertAttributeContains('test', 'table_name', $results[$count-1]);

	}

	/**
	 *
	 */
	public function testBaseGetByFilterLoadColumn()
	{

		$filter = array(
			'fields' => array(
				$this->table->getTableAlias() . '.id',
				$this->table->getTableAlias() . '.name',
			),
			'namePattern' => '%test%',
			'resultType' => 'column'
		);

		$stubRepository = $this->getStubbedRepository();
		$results = $stubRepository->getByFilter($filter);

		$this->assertTrue(is_array($results));
		$this->assertCount(7, $results); //only 2 field return
		foreach($results as $result){
			$this->assertTrue(is_numeric($result));
		}
	}

	/**
	 *
	 */
	public function testBaseGetByFilterLoadObject()
	{

		$filter = array(
			'fields' => array(
				$this->table->getTableAlias() . '.id',
				$this->table->getTableAlias() . '.name',
			),
			'namePattern' => '%test%',
			'resultType' => 'object'
		);

		$results = $this->repository->getByFilter($filter);
		$this->assertTrue($results instanceof \LMS\Libraries\Database\Object);
	}

	/**
	 *
	 */
	public function testBaseGetByFilter()
	{

		$filter = array(
			'fields' => array(
				$this->table->getTableAlias() . '.id',
				$this->table->getTableAlias() . '.name',
			),
			'id' => '1',
			'namePattern' => '%test%',
		);

		$stubRepository = $this->getStubbedRepository();
		$results = $stubRepository->getByFilter($filter);
		$this->assertCount(6, $results);
	}

	/**
	 *
	 */
	public function testGetTotal()
	{

		$filter = array(
			'namePattern' => '%test%',
		);

		$stubRepository = $this->getStubbedRepository();
		$count = $stubRepository->getTotal($filter);
		$this->assertEquals(7, $count);

		$filter = array(
			'fields' => array(
				$this->table->getTableAlias() . '.id',
				$this->table->getTableAlias() . '.name',
			),
			'namePattern' => '%test123123%',
		);

		$stubRepository = $this->getStubbedRepository();
		$count = $stubRepository->getTotal($filter);
		$this->assertEquals(0, $count);


		$filter = array(
			'fields' => array(
				$this->table->getTableAlias() . '.id',
				$this->table->getTableAlias() . '.name',
			),
			'namePattern' => '%test%',
		);

		$stubRepository = $this->getStubbedRepository();
		$count = $stubRepository->getTotal($filter);
		$this->assertEquals(7, $count);
	}



	/**
	 *
	 */
	public function testSaveArray()
	{
		$data = array(
			'table_name' => 'new_inserted_name',
			'key_id' => '3',
			'description' => 'new_inserted_desc',
			'date' => date('Y-m-d H:i:s')
		);

		$result = $this->repository->save($data);
		$this->assertTrue($result);

		$alias = $this->table->getTableAlias();
		$justInsertedObject = $this->repository->find(array($alias.'.*'), array('name = \'new_inserted_name\''));

		$this->assertCount(1, $justInsertedObject);

		foreach($data as $key=>$value){
			$this->assertAttributeEquals($value, $key, $justInsertedObject[0]);
		}
	}

	/**
	 * @throws \LMS\Libraries\Exception\Database\Error
	 */
	public function testSaveArrayWithRealTableName()
	{
		$data = array(
			'name' => 'new_inserted_name',
			'key_id' => '3',
			'description' => 'new_inserted_desc',
			'date' => date('Y-m-d H:i:s')
		);

		$result = $this->repository->save($data);
		$this->assertTrue($result);

		$alias = $this->table->getTableAlias();
		$justInsertedObject = $this->repository->find(array($alias.'.*'), array('name = \'new_inserted_name\''));

		$this->assertCount(1, $justInsertedObject);

		foreach($data as $key=>$value){
			$this->assertAttributeEquals($value, $this->table->getAlias($key), $justInsertedObject[0]);
		}
	}

	/**
	 *
	 */
	public function testSaveObject()
	{
		$data = array(
			'table_name' => 'new_inserted_name',
			'key_id' => '3',
			'description' => 'new_inserted_desc',
			'date' => date('Y-m-d H:i:s')
		);

		$result = $this->repository->save($data);
		$this->assertTrue($result);

		$alias = $this->table->getTableAlias();
		$justInsertedObject = $this->repository->find(array($alias.'.*'), array('name = \'new_inserted_name\''));

		$this->assertCount(1, $justInsertedObject);

		foreach($data as $key=>$value){
			$this->assertAttributeEquals($value, $key, $justInsertedObject[0]);
		}
	}

	/**
	 *
	 */
	public function testSaveWrongDataForBind()
	{
		$data = "wrong data";
		$this->setExpectedException('LMS\\Libraries\\Exception\\Database\\Error');
		$this->repository->save($data);
	}

	/**
	 *
	 */
	public function testSaveNonExistingField()
	{
		$data = array(
			'table_name' => 'new_inserted_name',
			'non_exists_field' => 'some data',
			'key_id' => '3',
			'description' => 'new_inserted_desc',
			'date' => date('Y-m-d H:i:s')
		);

		$result = $this->repository->save($data);
		$this->assertTrue($result);
	}

	/**
	 *
	 */
	public function testSaveMultiple()
	{
		$data = array(
			array(
				'table_name' => 'new_inserted_name_multiple',
				'key_id' => '3',
				'description' => 'new_inserted_desc',
				'date' => date('Y-m-d H:i:s')
			),
			array(
				'table_name' => 'new_inserted_name_multiple',
				'key_id' => '3',
				'description' => 'new_inserted_desc',
				'date' => date('Y-m-d H:i:s')
			),
			array(
				'table_name' => 'new_inserted_name_multiple',
				'key_id' => '3',
				'description' => 'new_inserted_desc',
				'date' => date('Y-m-d H:i:s')
			),
		);

		$result = $this->repository->saveMultiple($data);
		$this->assertTrue($result);

		$alias = $this->table->getTableAlias();
		$justInsertedObject = $this->repository->find(array($alias.'.*'), array('name = \'new_inserted_name_multiple\''));

		$this->assertCount(count($data), $justInsertedObject);

		foreach($data as $index=>$item){
			foreach($item as $key=>$value){
				$this->assertAttributeEquals($value, $key, $justInsertedObject[$index]);
			}
		}
	}

	/**
	 *
	 */
	public function testSaveMultipleWrongDataForBind()
	{
		$data = array(
			array(
				'table_name' => 'new_inserted_name_multiple',
				'key_id' => '3',
				'description' => 'new_inserted_desc',
				'date' => date('Y-m-d H:i:s')
			),
			'table_name' => 'new_inserted_name_multiple',
			array(
				'table_name' => 'new_inserted_name_multiple',
				'key_id' => '3',
				'description' => 'new_inserted_desc',
				'date' => date('Y-m-d H:i:s')
			),
		);

		$this->setExpectedException('LMS\\Libraries\\Exception\\Database\\Error');
		$result = $this->repository->saveMultiple($data);
	}

	/**
	 *
	 */
	public function testSaveMultipleEmpty()
	{
		$data = array();

		$result = $this->repository->saveMultiple($data);
		$this->assertFalse($result);
	}

	/**
	 *
	 */
	public function testSaveQueryExecutingError()
	{
		$data = array(
			'table_name' => 'new_inserted_name',
			'key_id' => '3',
			'description' => 'new_inserted_desc',
			'date' => date('Y-m-d H:i:s')
		);

		$this->dbo->disconnect();
		$this->setExpectedException('LMS\\Libraries\\Exception\\Database\\Error');
		$result = $this->repository->save($data);
	}

	/**
	 *
	 */
	public function testSaveMultipleQueryExecutingError()
	{
		$data = array(array(
			'table_name' => 'new_inserted_name',
			'key_id' => '3',
			'description' => 'new_inserted_desc',
			'date' => date('Y-m-d H:i:s')
		));

		$this->dbo->disconnect();
		$this->setExpectedException('LMS\\Libraries\\Exception\\Database\\Error');
		$result = $this->repository->saveMultiple($data);
	}



	/**
	 *
	 */
	public function testUpdate()
	{
		$data = array(
			'table_name' => 'updated_name',
			'key_id' => '3',
			'description' => 'updated_desc',
			'date' => date('Y-m-d H:i:s')
		);

		$result = $this->repository->update($data, array('id' => 1));
		$this->assertTrue($result);

		$alias = $this->table->getTableAlias();
		$justInsertedObject = $this->repository->find(array($alias.'.*'), array('id = 1'));

		$this->assertCount(1, $justInsertedObject);

		foreach($data as $key=>$value){
			$this->assertAttributeEquals($value, $key, $justInsertedObject[0]);
		}
	}

	public function testUpdateWithRealFieldName()
	{
		$data = array(
			'name' => 'updated_name',
			'key_id' => '3',
			'description' => 'updated_desc',
			'date' => date('Y-m-d H:i:s')
		);

		$result = $this->repository->update($data, array('id' => 1));
		$this->assertTrue($result);

		$alias = $this->table->getTableAlias();
		$justInsertedObject = $this->repository->find(array($alias.'.*'), array('id = 1'));

		$this->assertCount(1, $justInsertedObject);

		foreach($data as $key=>$value){
			$this->assertAttributeEquals($value, $this->table->getAlias($key), $justInsertedObject[0]);
		}
	}

	/**
	 *
	 */
	public function testUpdateWrongDataForBind()
	{
		$data = 'updated_name';
		$this->setExpectedException('LMS\\Libraries\\Exception\\Database\\Error');
		$this->repository->update($data, array('id' => 1));
	}

	/**
	 *
	 */
	public function testUpdateAll()
	{
		$data = array(
			'table_name' => 'updated_name',
			'key_id' => '3',
			'description' => 'updated_desc',
			'date' => date('Y-m-d H:i:s')
		);

		$result = $this->repository->update($data);
		$this->assertTrue($result);

		$alias = $this->table->getTableAlias();
		$justInsertedObject = $this->repository->find(array($alias.'.*'), array('name = \'updated_name\''));

		$this->assertCount(7, $justInsertedObject);

		foreach($data as $key=>$value){
			$this->assertAttributeEquals($value, $key, $justInsertedObject[0]);
		}
	}

	/**
	 *
	 */
	public function testUpdateWithMultipleWhere()
	{
		$data = array(
			'table_name' => 'updated_name',
			'key_id' => '3',
			'description' => 'updated_desc',
			'date' => date('Y-m-d H:i:s')
		);

		$idsArray = array(2, 3);
		$result = $this->repository->update($data, array('id' => $idsArray));
		$this->assertTrue($result);

		$alias = $this->table->getTableAlias();
		$justInsertedObject = $this->repository->find(array($alias.'.*'), array('name = \'updated_name\''));

		$this->assertCount(count($idsArray), $justInsertedObject);

		foreach($data as $key=>$value){
			foreach($justInsertedObject as $item){
				$this->assertAttributeEquals($value, $key, $item);
				$this->assertTrue(in_array($item->id, $idsArray));
			}
		}
	}

	/**
	 *
	 */
	public function testUpdateWrongSQL()
	{
		$data = array(
			'table_name' => 'updated_name',
			'key_id' => '3',
			'description' => 'updated_desc',
			'date' => date('Y-m-d H:i:s')
		);

		$this->setExpectedException('LMS\\Libraries\\Exception\\Database\\Error');
		$result = $this->repository->update($data, array('id2' => 1));
	}



	/**
	 *
	 */
	public function testDelete()
	{
		$result = $this->repository->delete(array('key_id' => 2));
		$this->assertTrue($result);

		$alias = $this->table->getTableAlias();
		$justDeletedObject = $this->repository->find(array($alias.'.*'), array('key_id = 2'));

		$this->assertCount(0, $justDeletedObject);
	}

	/**
	 *
	 */
	public function testDeleteAll()
	{
		$result = $this->repository->delete();
		$this->assertTrue($result);

		$alias = $this->table->getTableAlias();
		$justDeletedObject = $this->repository->find(array($alias.'.*'));

		$this->assertCount(0, $justDeletedObject);
	}

	/**
	 *
	 */
	public function testDeleteWithMultipleWhere()
	{
		$idsArray = array(2, 3);
		$result = $this->repository->delete(array('id' => $idsArray));
		$this->assertTrue($result);

		$alias = $this->table->getTableAlias();
		$justDeletedObject = $this->repository->find(array($alias.'.*'), array('id IN ('.implode(',', $idsArray).')'));

		$this->assertCount(0, $justDeletedObject);
	}

	/**
	 *
	 */
	public function testDeleteWithWrongDeleteCondition()
	{
		$this->setExpectedException('LMS\\Libraries\\Exception\\Database\\Error');
		$this->repository->delete(array('id' => ''));
	}

	/**
	 *
	 */
	public function testDeleteWrongSQL()
	{
		$this->setExpectedException('LMS\\Libraries\\Exception\\Database\\Error');
		$result = $this->repository->delete(array('id2' => 1));
	}



	/**
	 * @depends testSaveArray
	 * @depends testSimpleFind
	 */
	public function testLastInsertId()
	{
		$this->repository->save(array(
			'table_name' => 'new_inserted_name',
			'key_id' => '3',
			'description' => 'new_inserted_desc',
			'date' => date('Y-m-d H:i:s')
		));

		$lastInsertId = $this->repository->lastInsertedID();
		$realLastInsertIdData = $this->repository->find('MAX('.$this->table->getTableAlias().'.id) as id');
		$realLastInsertId = $realLastInsertIdData[0]->id;

		$this->assertEquals($realLastInsertId, $lastInsertId);
	}


	/**
	 *
	 */
	public function testBeforeSave()
	{
		$stubRepository = $this->getMockBuilder('\LMS\Tests\Data\Mocks\RepositoryMock')
			->setMethods(array('beforeSave'))
			->setConstructorArgs(array(array('db' => $this->table->getDbo())))
			->getMock();

		$stubRepository->expects($this->once())
						->method('beforeSave')
						->with($this->isInstanceOf('stdClass'));

		$data = array(
			'table_name' => 'new_inserted_name',
			'key_id' => '3',
			'description' => 'new_inserted_desc',
			'date' => date('Y-m-d H:i:s')
		);
		$stubRepository->save((object)$data);
	}

	/**
	 *
	 */
	public function testBeforeSaveMultiple()
	{
		$stubRepository = $this->getMockBuilder('\LMS\Tests\Data\Mocks\RepositoryMock')
			->setMethods(array('beforeSaveMultiple'))
			->setConstructorArgs(array(array('db' => $this->table->getDbo())))
			->getMock();

		$stubRepository->expects($this->once())
			->method('beforeSaveMultiple')
			->with($this->isType('array'));

		$data = array(array(
			'table_name' => 'new_inserted_name',
			'key_id' => '3',
			'description' => 'new_inserted_desc',
			'date' => date('Y-m-d H:i:s')
		));
		$stubRepository->saveMultiple($data);
	}

	/**
	 *
	 */
	public function testBeforeUpdate()
	{
		$stubRepository = $this->getMockBuilder('\LMS\Tests\Data\Mocks\RepositoryMock')
			->setMethods(array('beforeUpdate'))
			->setConstructorArgs(array(array('db' => $this->table->getDbo())))
			->getMock();

		$stubRepository->expects($this->once())
			->method('beforeUpdate')
			->with($this->isType('array'), $this->isType('array'));

		$data = array(
			'table_name' => 'new_inserted_name',
			'key_id' => '3',
			'description' => 'new_inserted_desc',
			'date' => date('Y-m-d H:i:s')
		);
		$stubRepository->update($data);
	}

	/**
	 *
	 */
	public function testBeforeDelete()
	{
		$stubRepository = $this->getMockBuilder('\LMS\Tests\Data\Mocks\RepositoryMock')
			->setMethods(array('beforeDelete'))
			->setConstructorArgs(array(array('db' => $this->table->getDbo())))
			->getMock();

		$stubRepository->expects($this->once())
			->method('beforeDelete')
			->with($this->isType('array'));

		$data = array(
			'id' => 1,
		);
		$stubRepository->delete($data);
	}



	private function getStubbedRepository()
	{
		$stubRepository = $this->getMockBuilder('\LMS\Libraries\Base\Repository')
			->setMethods(array('prepareFilter'))
			->setConstructorArgs(array(array('db' => $this->table->getDbo())))
			->getMock();

		$stubRepository->expects($this->once())
			->method('prepareFilter')
			->will($this->returnCallback(function($query, $filterOptions){
				if(isset($filterOptions['id'])){
					$alias = $query->getTable()->getTableAlias();
					$query->where($alias.'.id > '.$filterOptions['id']);
				}

				if(isset($filterOptions['namePattern'])){
					$alias = $query->getTable()->getTableAlias();
					$query->where($alias.'.name LIKE \''.$filterOptions['namePattern'].'\'');
				}
				return $query;
			}));

		$stubRepository->setTable($this->table);
		return $stubRepository;
	}

}