<?php
/**
 * User: nester_a
 * Date: 25.11.13
 */

namespace LMS\Tools\Conference\Response;

use LMS\Libraries\Cache;
use LMS\Libraries\Date;

/**
 * Class ResponseProcessor
 *
 * @method renderStart
 * @method renderStartNoAvailableConference
 * @method renderRoom
 * @method renderEnterRoomError
 * @method renderUserAccess
 * @method renderArchive
 * @method renderRecordEdit
 * @method renderBooking
 * @method renderBookingNew
 * @method renderBookingEdit
 */
class Processor
{
    protected $renderParams = array();
    protected $cacheDriver;
    protected $config;
    protected $templateDir;
	protected $schemaDir;
    protected $enableCaching = true;
	protected $dateHandler;

    public function __construct($diContainer)
    {
        $this->cacheDriver = $diContainer['cacheDriver'];
        $this->config = $diContainer['config'];
		$this->dateHandler = $diContainer['dateHandler'];
    }


    public function __call($method, $arguments)
    {
        if(strpos($method, 'render') !== false){
            $templateName = strtolower(substr($method, 6));
            return $this->render($templateName);
        }

        throw new \InvalidArgumentException('There is no method ' . $method);
    }


    public function setTemplateDir($dir)
    {
        $this->templateDir = $dir;
    }

	public function setSchemaDir($dir)
	{
		$this->schemaDir = $dir;
	}

    public function setRenderParams($params)
    {
        $this->renderParams = $params;
    }

    public function addRenderParams($params)
    {
        $this->renderParams = array_merge($this->renderParams, $params);
    }

    public function clearRenderParams()
    {
        $this->renderParams = array();
    }



    public function setCaching($enableCaching)
    {
        $this->enableCaching = $enableCaching;
    }



    public function render($templateName)
    {
        $response = new Response($this->_doRender($this->templateDir . $templateName .'.phtml', $this->renderParams, $this->enableCaching));
        return $response;
    }


    public function operationOk($message)
    {
        return new Response(
            $message,
            Response::STATUS_OK,
            Response::TYPE_OPERATION_RESULT
        );
    }



    public function jsonError($data)
    {
        return new Response(
            json_encode(is_array($data) ? $data : array($data)),
            Response::STATUS_ERROR,
            Response::TYPE_JSON_DATA
        );
    }

    public function jsonOk($data)
    {
        return new Response(
            json_encode(is_array($data) ? $data : array($data)),
            Response::STATUS_OK,
            Response::TYPE_JSON_DATA
        );
    }



    public function rawDataOk($data)
    {
        return new Response(
            $data,
            Response::STATUS_OK,
            Response::TYPE_DATA
        );
    }


    public function xmlOk($data)
    {
        return new Response(
            $data,
            Response::STATUS_OK,
            Response::TYPE_XML_DATA
        );
    }


    public function file($srvName,$headers)
    {
        return new Response(
            $srvName,
            Response::STATUS_OK,
            Response::TYPE_FILE,
            $headers
        );
    }

    public function LMSfile($fileID)
    {
        $headers = array();
        $headers['Content-Disposition'] = 'inline';

        return new Response(
            $fileID,
            Response::STATUS_OK,
            Response::TYPE_LMS_FILE,
            $headers
        );
    }




    public function detectUserBrowser($agent)
    {
        if (preg_match('/Opera(\/| )([0-9].[0-9]{1,2})/', $agent)) {
            $UserBrowser = "Opera";
        }
        elseif (preg_match('/MSIE ([0-9].[0-9]{1,2})/', $agent)) {
            $UserBrowser = "IE";
        } else {
            $UserBrowser = '';
        }

        return $UserBrowser;
    }



    public function getFileMimeType($userBrowser)
    {
        return ($userBrowser == 'IE' || $userBrowser == 'Opera') ? 'application/octetstream' : 'application/octet-stream';
    }

    public function getHeadersForDisplayingFile($userBrowser, $mimeType, $fileName, $fileSize)
    {
        $headers = array();
        $headers['Content-Type'] = $mimeType;
        $headers['Expires'] = date('D, d M Y H:i:s') . ' GMT';
        $headers['Content-Disposition'] = 'inline; filename="' . $fileName . '"';
        $headers['Content-Length'] = $fileSize;

        if ($userBrowser == 'IE') {
            $headers['Cache-Control'] = 'must-revalidate, post-check=0, pre-check=0';
            $headers['Pragma'] = 'public';
        } else {
            $headers['Pragma'] = 'no-cache';
        }
        return $headers;
    }

    /**
     * @param $filename
     * @return array
     */
    public function getHeadersForDownloadFile($filename)
    {
        $headers = array();
        $headers['Content-Length'] = filesize($filename);
        $headers["Content-disposition"]= "attachment; filename=" . basename($filename);
        return $headers;
    }


	public function loadXMLSchema($filename, $params = array())
	{
		return $this->_doRender($this->schemaDir . $filename, $params);
	}


    protected function _doRender($filename, $params = array(), $enableCaching = true)
    {
        extract($params);

        $cacheHelper = Cache\Helper::getInstance();
        $cacheHelper->setCacheDriver($this->cacheDriver);
        $cacheHelper->setConfig($this->config);

        if(!$enableCaching){
            $cacheHelper->disableCaching();
        }

        ob_start();
        include_once $filename;
        $template = ob_get_contents();
        ob_end_clean();

        return $template;
    }

}