<?php

namespace LMS\Tools\Conference\Service;

use LMS\Libraries\Exception\Operation;
use LMS\Libraries\Exception\Validation;
use LMS\Tools\Conference\Repositories;
use LMS\Libraries\Base;
use LMS\Libraries;


class Booking extends Base\Service
{

	/**
	 * @var Libraries\Date\Handler
	 */
	protected $dateHandler;

	/**
	 * @var Repositories\Booking
	 */
	protected $repository;

	public function __construct($options)
	{
		parent::__construct($options);
		$this->repository = isset($options['repository']) ? $options['repository'] : new Repositories\Booking(array('db' => Libraries\ServiceLocator::get('db')));
		$this->dateHandler = new Libraries\Date\Handler();
	}


	/**
	 * initialize new booking
	 *
	 * @param $userIdentity
	 * @return \stdClass
	 */
	public function initBooking($userIdentity)
	{
		$row = new \stdClass();
		$row->p_id = 0;
		$row->public = 0;
		$row->cur_date = $this->dateHandler->getCurrentDatetime("Y-m-d");
		$row->p_name = '';
		$row->p_description = '';
		$row->teacher_id = $userIdentity->id;
		return $row;
	}



	/**
	 * @param $courseID
	 * @param $myIdentity
	 * @return bool
	 */
	public function isAvailableConferenceExists($courseID, $myIdentity)
	{
		$availableConferences = $this->repository->getAvailable($courseID, $this->container['config']->get('offset'));

		if (count($availableConferences) > 0){
			foreach($availableConferences as $conference){
				if (!$conference->public)
				{
					$userModel = new Repositories\User(array('db' => $this->container['db']));
					$allowedUserIDs = $userModel->getAllowedToAccess($availableConferences[0]->p_id);
					if(in_array($myIdentity->id, $allowedUserIDs) || $conference->teacher_id == $myIdentity->id){
						return true;
					}
				}else{
					return true;
				}
			}
		}

		return false;
	}

	/**
	 * @param array $filterOptions
	 * @return \stdClass
	 */
	public function fetchData($filterOptions)
	{
		$bookings = new \stdClass();

		$bookings->total = $this->repository->count($filterOptions['courseID'], $filterOptions['filterTeacher'], $filterOptions['filterStudent'], $filterOptions);

		$filterOptions['offset'] = $filterOptions['limitStart'];

		$bookings->rows = $this->repository->getByFilter($filterOptions);
		$bookings->limit = $filterOptions['limit'];
		$bookings->limitStart = $filterOptions['limitStart'];
		$bookings->teacher = $filterOptions['filterTeacher'];
		$bookings->stundet = $filterOptions['filterStudent'];

		return $bookings;
	}


	/**
	 * @param $bookings
	 */
	public function getRequery($bookings, $timeOffset)
	{
		$isFound = $this->isContainsAvailableBookedConference($bookings->rows, $timeOffset);

		if (!$isFound) {
			$doSearch = !$this->isEndOfBookedConferencesList($bookings->limitStart, $bookings->limit, $bookings->total);
			while ($doSearch) {
				$bookings->limitStart += $bookings->limit;
				$rowsToCheck = $this->repository->getByFilter($bookings->courseID);
				$isFound = $this->isContainsAvailableBookedConference($rowsToCheck, $timeOffset);
				$doSearch = !$this->isEndOfBookedConferencesList($bookings->limitStart, $bookings->limit, $bookings->total) && !$isFound;
			}
			$bookings->rows = isset($rowsToCheck) && $isFound ? $rowsToCheck : $bookings->rows;
		}

		return $bookings;
	}


	/**
	 * @param $data
	 * @return array
	 * @throws Operation\Error
	 */
	public function prepareMultipleBookings($data)
	{
		$fromTime = $data['from_hour'];
		$toTime = $data['to_hour'];
		$fromMinutes = $data['from_min'];
		$toMinutes =  $data['to_min'];
		$weekDay = $data['weekDay'];
		$monthDay = $data['monthDay'];

		$dateService = new Date();

		$preparedData = array();

		if (count($weekDay) && count($monthDay)){
			foreach ($monthDay as $monthYear){
				$allDates = $dateService->prepareDates($monthYear, $weekDay);
				if(!count($allDates)){
					continue;
				}

				foreach ($allDates as $date){
					$data['from_time'] = mktime($fromTime,$fromMinutes,0,substr($date,5,2),substr($date,8,2),substr($date,0,4));
					$data['to_time'] = mktime($toTime,$toMinutes,0,substr($date,5,2),substr($date,8,2),substr($date,0,4));

					$this->validate($data);
					$preparedData[] = $data;
				}
			}
		}
		return $preparedData;
	}


	public function validate($data)
	{
		$courseID = $data['courseID'];
		$teacherID =  $data['teacher_id'];
		$bookedConferenceID = $data['bookedConferenceID'];

		$currentTimestamp = $this->dateHandler->getCurrentTimestamp();

		if(($data['from_time'] >= $data['to_time']) || $data['from_time'] < $currentTimestamp){
			throw new Validation\Error(
				'Please provide correct conference period',
				sefRelToAbs('index.php?option=com_joomla_lms&Itemid='.$data['Itemid'].'&task=conference.booking&id='.$courseID)
			);
		}

		if ($this->repository->isAlreadyExistsForPeriod($courseID, $teacherID, $bookedConferenceID, $data['from_time'], $data['to_time'])) {
			throw new Validation\Error(
				'Conference for this period already exists',
				sefRelToAbs('index.php?option=com_joomla_lms&Itemid='.$data['Itemid'].'&task=conference.booking&id='.$courseID)
			);
		}

	}


	/**
	 * Determine whether or not $rows contains available bookings
	 *
	 * @param array $rows
	 * @return bool
	 */
	protected function isContainsAvailableBookedConference($rows, $offset)
	{
		foreach ($rows as $row) {
			if ($row->from_time >= ( time() + $offset * 60 * 60 )) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Determine whether we processed all bookings
	 *
	 * @param int $diContainer['limitStart']
	 * @param int $diContainer['limit']
	 * @param int $total
	 * @return bool
	 */
	protected function isEndOfBookedConferencesList($limitStart, $limit, $total)
	{
		return ($limitStart + $limit > $total);
	}


}